export var isNull = function isNull(value) {
  return value === undefined || value === null;
};

export var isFunction = function isFunction(value) {
  return typeof value === 'function';
};

export var isObject = function isObject(value) {
  return value !== null && Object.prototype.toString.call(value) === '[object Object]';
};

export var getComputedStyle = window.getComputedStyle || document.defaultView.getComputedStyle;

export var getNodeSize = function getNodeSize(node) {
  if (isNull(node)) return null;
  return {
    width: node.offsetWidth,
    height: node.offsetHeight
  };
};

export var getDisplayName = function getDisplayName(component) {
  if (isNull(component)) return null;
  return component.displayName || component.name || (typeof component === 'string' ? component : 'Unknown');
};

export var addHandler = window.addEventListener ? function (target, type, handler) {
  target.addEventListener(type, handler, false);
} : function (target, type, handler) {
  target.attachEvent('on' + type, handler);
};

export var removeHandler = window.removeEventListener ? function (target, type, handler) {
  target.removeEventListener(type, handler, false);
} : function (target, type, handler) {
  target.detachEvent('on' + type, handler);
};

export var warn = function warn(msg) {
  if (isNull(msg)) return;
  if (process.env.NODE_ENV !== 'production') {
    throw new Error(msg);
  } else if (console) {
    console.warn('[react-hold]', msg);
  }
};