'use strict';

exports.__esModule = true;
exports.symbol = undefined;

exports.default = function (component) {
  if (component[symbol]) return component[symbol];

  var componentOriginalMethods = null;

  var replaceLifecycleMethods = function replaceLifecycleMethods() {
    var componentDidMount = void 0;
    var componentWillUnmount = void 0;
    var prototype = component.prototype;
    if ((0, _utils.isNull)(componentOriginalMethods) && prototype) {
      if ((0, _utils.isFunction)(prototype.componentDidMount)) {
        componentDidMount = prototype.componentDidMount;
        prototype.componentDidMount = null;
      }
      if ((0, _utils.isFunction)(prototype.componentWillUnmount)) {
        componentWillUnmount = prototype.componentWillUnmount;
        prototype.componentWillUnmount = function willUnmount() {
          try {
            componentWillUnmount.call(this);
          } catch (e) {
            // swallow exception
          }
        };
      }
      componentOriginalMethods = {
        componentDidMount: componentDidMount,
        componentWillUnmount: componentWillUnmount
      };
    }
  };

  var restoreLifecycleMethods = function restoreLifecycleMethods() {
    var methods = componentOriginalMethods;
    if (!(0, _utils.isNull)(methods) && component.prototype) {
      Object.keys(methods).forEach(function (name) {
        if ((0, _utils.isFunction)(methods[name])) {
          component.prototype[name] = methods[name];
        }
      });
      componentOriginalMethods = null;
    }
  };

  var refiter = {
    refit: replaceLifecycleMethods,
    undo: restoreLifecycleMethods
  };

  if (typeof component !== 'string') {
    component[symbol] = refiter;
  }

  return refiter;
};

var _utils = require('./utils');

var symbol = exports.symbol = Symbol ? Symbol('react-hold/refiter') : '$$react-hold/refiter';

/**
 * Create APIs to refit the component lifecycle methods.
 *
 * @param {Component|string} component
 * @returns {{refit, undo}}
 */